@@ -143,12 +143,17 @@ struct drumkv1_env
 {
 	// envelope stages
 
-	enum Stage { Attack = 0, Decay1, Decay2, Done };
+	enum Stage { Idle = 0, Attack, Decay1, Decay2 };
 
 	// per voice
 
 	struct State
 	{
+		// ctor.
+		State() : running(false), stage(Idle),
+			phase(0.0f), delta(0.0f), value(0.0f),
+			c1(1.0f), c0(0.0f), frames(0) {}
+
 		// process
 		float tick()
 		{
@@ -211,7 +216,7 @@ struct drumkv1_env
 		}
 		else if (p->stage == Decay2) {
 			p->running = false;
-			p->stage = Done;
+			p->stage = Idle;
 			p->frames = 0;
 			p->phase = 0.0f;
 			p->delta = 0.0f;
@@ -221,19 +226,6 @@ struct drumkv1_env
 		}
 	}
 
-	void note_on(State *p)
-	{
-		p->running = true;
-		p->stage = Attack;
-		p->frames = uint32_t(*attack * *attack * max_frames);
-		if (p->frames < min_frames) // prevent click on too fast attack
-			p->frames = min_frames;
-		p->phase = 0.0f;
-		p->delta = 1.0f / float(p->frames);
-		p->c1 = 1.0f - p->value;
-		p->c0 = p->value;
-	}
-
 	void note_off(State *p)
 	{
 		p->running = true;
@@ -251,8 +243,7 @@ struct drumkv1_env
 	{
 		p->running = true;
 		p->stage = Decay2;
-		if (p->frames > min_frames)
-			p->frames = min_frames;
+		p->frames = min_frames;
 		p->phase = 0.0f;
 		p->delta = 1.0f / float(p->frames);
 		p->c1 = -(p->value);
@@ -373,6 +364,7 @@ struct drumkv1_def
 	float *modwheel;
 	float *pressure;
 	float *velocity;
+	float *channel;
 	float *noteoff;
 };
 
@@ -699,13 +691,14 @@ struct drumkv1_voice : public drumkv1_list<drumkv1_voice>
 
 	drumkv1_elem *elem;
 
-	drumkv1_generator  gen1;
-	drumkv1_oscillator lfo1;
-
 	int note;									// voice note
 	int group;									// voice group
-	float vel;									// velocity to vol
-	float pre;									// key pressure/aftertouch
+
+	float vel;									// key velocity
+	float pre;									// key pressure/after-touch
+
+	drumkv1_generator  gen1;
+	drumkv1_oscillator lfo1;
 
 	float gen1_freq;							// frequency and phase
 
@@ -1099,6 +1092,7 @@ void drumkv1_impl::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 	case drumkv1::DEF1_MODWHEEL:  m_def.modwheel  = pfParam; break;
 	case drumkv1::DEF1_PRESSURE:  m_def.pressure  = pfParam; break;
 	case drumkv1::DEF1_VELOCITY:  m_def.velocity  = pfParam; break;
+	case drumkv1::DEF1_CHANNEL:   m_def.channel   = pfParam; break;
 	case drumkv1::DEF1_NOTEOFF:   m_def.noteoff   = pfParam; break;
 	case drumkv1::CHO1_WET:       m_cho.wet       = pfParam; break;
 	case drumkv1::CHO1_DELAY:     m_cho.delay     = pfParam; break;
@@ -1139,6 +1133,7 @@ float *drumkv1_impl::paramPort ( drumkv1::ParamIndex index )
 	case drumkv1::DEF1_MODWHEEL:  pfParam = m_def.modwheel;  break;
 	case drumkv1::DEF1_PRESSURE:  pfParam = m_def.pressure;  break;
 	case drumkv1::DEF1_VELOCITY:  pfParam = m_def.velocity;  break;
+	case drumkv1::DEF1_CHANNEL:   pfParam = m_def.channel;   break;
 	case drumkv1::DEF1_NOTEOFF:   pfParam = m_def.noteoff;   break;
 	case drumkv1::CHO1_WET:       pfParam = m_cho.wet;       break;
 	case drumkv1::CHO1_DELAY:     pfParam = m_cho.delay;	 break;
@@ -1180,6 +1175,15 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	if (size < 2)
 		return;
 
+	// channel filter
+	const int channel = (data[0] & 0x0f) + 1;
+
+	const int ch = int(*m_def.channel);
+	const int on = (ch == 0 || ch == channel);
+
+	if (!on)
+		return;
+
 	// note on
 	const int status = (data[0] & 0xf0);
 	const int key    = (data[1] & 0x7f);
@@ -1204,8 +1208,8 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			elem->dcf1.env.note_off_fast(&pv->dcf1_env);
 			elem->lfo1.env.note_off_fast(&pv->lfo1_env);
 			elem->dca1.env.note_off_fast(&pv->dca1_env);
-			pv->note = -1;
 			m_notes[key] = 0;
+			pv->note = -1;
 		}
 		// find free voice
 		pv = alloc_voice(key);
@@ -1569,7 +1573,7 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 			if (pv->dca1_env.running && pv->dca1_env.frames == 0)
 				elem->dca1.env.next(&pv->dca1_env);
 
-			if (pv->dca1_env.stage == drumkv1_env::Done || pv->gen1.isOver()) {
+			if (pv->dca1_env.stage == drumkv1_env::Idle || pv->gen1.isOver()) {
 				if (pv->note >= 0)
 					m_notes[pv->note] = 0;
 				if (pv->group >= 0 && m_group[pv->group] == pv)
