@@ -20,6 +20,7 @@
 *****************************************************************************/
 
 #include "drumkv1widget.h"
+#include "drumkv1_param.h"
 
 #include <QDomDocument>
 #include <QTextStream>
@@ -29,81 +30,6 @@
 #include <QDir>
 
 
-//-------------------------------------------------------------------------
-// default state (params)
-
-static
-struct {
-
-	const char *name;
-	float value;
-
-} drumkv1_default_params[drumkv1::NUM_PARAMS] = {
-
-	{ "GEN1_SAMPLE",   36.0f }, // Bass Drum 1 (GM) aka. C2 (36)
-	{ "GEN1_GROUP",     0.0f },
-	{ "GEN1_COARSE",    0.0f },
-	{ "GEN1_FINE",      0.0f },
-	{ "DCF1_CUTOFF",    1.0f }, // 0.5f
-	{ "DCF1_RESO",      0.0f },
-	{ "DCF1_TYPE",      0.0f },
-	{ "DCF1_SLOPE",     0.0f },
-	{ "DCF1_ENVELOPE",  1.0f },
-	{ "DCF1_ATTACK",    0.0f },
-	{ "DCF1_DECAY1",    0.5f },
-	{ "DCF1_LEVEL2",    0.2f },
-	{ "DCF1_DECAY2",    0.5f },
-	{ "LFO1_SHAPE",     1.0f },
-	{ "LFO1_WIDTH",     1.0f },
-	{ "LFO1_RATE",      0.5f },
-	{ "LFO1_SWEEP",     0.0f },
-	{ "LFO1_PITCH",     0.0f },
-	{ "LFO1_CUTOFF",    0.0f },
-	{ "LFO1_RESO",      0.0f },
-	{ "LFO1_PANNING",   0.0f },
-	{ "LFO1_VOLUME",    0.0f },
-	{ "LFO1_ATTACK",    0.0f },
-	{ "LFO1_DECAY1",    0.5f },
-	{ "LFO1_LEVEL2",    0.2f },
-	{ "LFO1_DECAY2",    0.5f },
-	{ "DCA1_VOLUME",    0.5f },
-	{ "DCA1_ATTACK",    0.0f },
-	{ "DCA1_DECAY1",    0.5f },
-	{ "DCA1_LEVEL2",    0.2f },
-	{ "DCA1_DECAY2",    0.5f },
-	{ "OUT1_WIDTH",     0.0f },
-	{ "OUT1_PANNING",   0.0f },
-	{ "OUT1_VOLUME",    0.5f },
-
-	{ "DEF1_PITCHBEND", 0.2f },
-	{ "DEF1_MODWHEEL",  0.2f },
-	{ "DEF1_PRESSURE",  0.2f },
-	{ "DEF1_VELOCITY",  0.2f },
-	{ "DEF1_NOTEOFF",   1.0f },
-
-	{ "CHO1_WET",       0.0f },
-	{ "CHO1_DELAY",     0.5f },
-	{ "CHO1_FEEDB",     0.5f },
-	{ "CHO1_RATE",      0.5f },
-	{ "CHO1_MOD",       0.5f },
-	{ "FLA1_WET",       0.0f },
-	{ "FLA1_DELAY",     0.5f },
-	{ "FLA1_FEEDB",     0.5f },
-	{ "FLA1_DAFT",      0.0f },
-	{ "PHA1_WET",       0.0f },
-	{ "PHA1_RATE",      0.5f },
-	{ "PHA1_FEEDB",     0.5f },
-	{ "PHA1_DEPTH",     0.5f },
-	{ "PHA1_DAFT",      0.0f },
-	{ "DEL1_WET",       0.0f },
-	{ "DEL1_DELAY",     0.5f },
-	{ "DEL1_FEEDB",     0.5f },
-	{ "DEL1_BPM",     180.0f },
-	{ "DYN1_COMPRESS",  0.0f },
-	{ "DYN1_LIMIT",     1.0f }
-};
-
-
 //-------------------------------------------------------------------------
 // drumkv1widget - impl.
 //
@@ -137,7 +63,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
-		m_params_ab[i] = drumkv1_default_params[i].value;
+		m_params_ab[i] = drumkv1_param::paramDefaultValue(drumkv1::ParamIndex(i));
 
 	// Start clean.
 	m_iUpdate = 0;
@@ -178,6 +104,14 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.Dcf1SlopeKnob->insertItems(0, slopes);
 
+	// Channel filters
+	QStringList channels;
+	channels << tr("Omni");
+	for (int iChannel = 0; iChannel < 16; ++iChannel)
+		channels << QString::number(iChannel + 1);
+
+	m_ui.Def1ChannelKnob->insertItems(0, channels);
+
 	// Noteoff modes.
 	QStringList modes;
 	modes << tr("Disabled");
@@ -404,6 +338,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(drumkv1::DEF1_MODWHEEL,  m_ui.Def1ModwheelKnob);
 	setParamKnob(drumkv1::DEF1_PRESSURE,  m_ui.Def1PressureKnob);
 	setParamKnob(drumkv1::DEF1_VELOCITY,  m_ui.Def1VelocityKnob);
+	setParamKnob(drumkv1::DEF1_CHANNEL,   m_ui.Def1ChannelKnob);
 	setParamKnob(drumkv1::DEF1_NOTEOFF,   m_ui.Def1NoteoffKnob);
 
 	// OUT1
@@ -565,11 +500,17 @@ void drumkv1widget::paramChanged ( float fValue )
 // Reset all param knobs to default values.
 void drumkv1widget::resetParams (void)
 {
+	drumkv1 *pDrumk = instance();
+	if (pDrumk == NULL)
+		return;
+
+	pDrumk->reset();
+
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
 		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-		float fValue = drumkv1_default_params[i].value;
+		float fValue = drumkv1_param::paramDefaultValue(index);
 		drumkv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
 			fValue = pKnob->defaultValue();
@@ -653,7 +594,7 @@ void drumkv1widget::resetParamValues ( uint32_t nparams )
 
 	for (uint32_t i = 0; i < nparams; ++i) {
 		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-		float fValue = drumkv1_default_params[i].value;
+		float fValue = drumkv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
@@ -725,8 +666,10 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 
 	static QHash<QString, drumkv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
-		for (uint32_t i = drumkv1::NUM_ELEMENT_PARAMS; i < drumkv1::NUM_PARAMS; ++i)
-			s_hash.insert(drumkv1_default_params[i].name, drumkv1::ParamIndex(i));
+		for (uint32_t i = drumkv1::NUM_ELEMENT_PARAMS; i < drumkv1::NUM_PARAMS; ++i) {
+			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+			s_hash.insert(drumkv1_param::paramName(index), index);
+		}
 	}
 
 	clearElements();
@@ -752,7 +695,7 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 				if (eChild.isNull())
 					continue;
 				if (eChild.tagName() == "elements") {
-					loadElements(instance(), eChild);
+					drumkv1_param::loadElements(instance(), eChild);
 				}
 				else
 				if (eChild.tagName() == "params") {
@@ -811,16 +754,17 @@ void drumkv1widget::savePreset ( const QString& sFilename )
 	ePreset.setAttribute("version", DRUMKV1_VERSION);
 
 	QDomElement eElements = doc.createElement("elements");
-	saveElements(instance(), doc, eElements);
+	drumkv1_param::saveElements(instance(), doc, eElements);
 	ePreset.appendChild(eElements);
 
 	QDomElement eParams = doc.createElement("params");
 	for (uint32_t i = drumkv1::NUM_ELEMENT_PARAMS; i < drumkv1::NUM_PARAMS; ++i) {
 		QDomElement eParam = doc.createElement("param");
+		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		eParam.setAttribute("index", QString::number(i));
-		eParam.setAttribute("name", drumkv1_default_params[i].name);
-		eParam.appendChild(doc.createTextNode(QString::number(
-			paramValue(drumkv1::ParamIndex(i)))));
+		eParam.setAttribute("name", drumkv1_param::paramName(index));
+		eParam.appendChild(
+			doc.createTextNode(QString::number(paramValue(index))));
 		eParams.appendChild(eParam);
 	}
 	ePreset.appendChild(eParams);
@@ -837,108 +781,6 @@ void drumkv1widget::savePreset ( const QString& sFilename )
 }
 
 
-// Element serialization methods.
-void drumkv1widget::loadElements (
-	drumkv1 *pDrumk, const QDomElement& eElements,
-	const drumkv1_map_path& mapPath )
-{
-	if (pDrumk == NULL)
-		return;
-
-	static QHash<QString, drumkv1::ParamIndex> s_hash;
-	if (s_hash.isEmpty()) {
-		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i)
-			s_hash.insert(drumkv1_default_params[i].name, drumkv1::ParamIndex(i));
-	}
-
-	for (QDomNode nElement = eElements.firstChild();
-			!nElement.isNull();
-				nElement = nElement.nextSibling()) {
-		QDomElement eElement = nElement.toElement();
-		if (eElement.isNull())
-			continue;
-		if (eElement.tagName() == "element") {
-			int note = eElement.attribute("index").toInt();
-			drumkv1_element *element = pDrumk->addElement(note);
-			for (QDomNode nChild = eElement.firstChild();
-					!nChild.isNull();
-						nChild = nChild.nextSibling()) {
-				QDomElement eChild = nChild.toElement();
-				if (eChild.isNull())
-					continue;
-				if (eChild.tagName() == "sample") {
-				//	int index = eChild.attribute("index").toInt();
-					const QString& sFilename = eChild.text();
-					element->setSampleFile(
-						mapPath.absolutePath(sFilename).toUtf8().constData());
-				}
-				else
-				if (eChild.tagName() == "params") {
-					for (QDomNode nParam = eChild.firstChild();
-							!nParam.isNull();
-								nParam = nParam.nextSibling()) {
-						QDomElement eParam = nParam.toElement();
-						if (eParam.isNull())
-							continue;
-						if (eParam.tagName() == "param") {
-							drumkv1::ParamIndex index = drumkv1::ParamIndex(
-								eParam.attribute("index").toULong());
-							const QString& sName = eParam.attribute("name");
-							if (!sName.isEmpty() && s_hash.contains(sName))
-								index = s_hash.value(sName);
-							float fValue = eParam.text().toFloat();
-							element->setParamValue(index, fValue);
-						}
-					}
-				}
-			}
-		}
-	}
-
-	pDrumk->reset();
-}
-
-
-void drumkv1widget::saveElements (
-	drumkv1 *pDrumk, QDomDocument& doc, QDomElement& eElements,
-	const drumkv1_map_path& mapPath )
-{
-	if (pDrumk == NULL)
-		return;
-
-	for (int note = 0; note < 128; ++note) {
-		drumkv1_element *element = pDrumk->element(note);
-		if (element == NULL)
-			continue;
-		const char *pszSampleFile = element->sampleFile();
-		if (pszSampleFile == NULL)
-			continue;
-		QDomElement eElement = doc.createElement("element");
-		eElement.setAttribute("index", QString::number(note));
-		eElement.setAttribute("name", noteName(note));
-		QDomElement eSample = doc.createElement("sample");
-		eSample.setAttribute("index", 0);
-		eSample.setAttribute("name", "GEN1_SAMPLE");
-		eSample.appendChild(doc.createTextNode(
-			mapPath.abstractPath(pszSampleFile)));
-		eElement.appendChild(eSample);
-		QDomElement eParams = doc.createElement("params");
-		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			QDomElement eParam = doc.createElement("param");
-			eParam.setAttribute("index", QString::number(i));
-			eParam.setAttribute("name", drumkv1_default_params[i].name);
-			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			float *pfParam = element->paramPort(index);
-			eParam.appendChild(doc.createTextNode(QString::number(
-				pfParam ? *pfParam : element->paramValue(index))));
-			eParams.appendChild(eParam);
-		}
-		eElement.appendChild(eParams);
-		eElements.appendChild(eElement);
-	}
-}
-
-
 // Sample reset slot.
 void drumkv1widget::clearSample (void)
 {
@@ -1001,7 +843,7 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 		element = pDrumk->addElement(note);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			float fValue = drumkv1_default_params[i].value;
+			float fValue = drumkv1_param::paramDefaultValue(index);
 			element->setParamValue(index, fValue);
 		}
 		pDrumk->setCurrentElement(note);
@@ -1203,7 +1045,7 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 		element = pDrumk->addElement(iCurrentNote);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			float fValue = drumkv1_default_params[i].value;
+			float fValue = drumkv1_param::paramDefaultValue(index);
 			element->setParamValue(index, fValue, 0);
 			element->setParamValue(index, fValue);
 		}
